@@ -2,8 +2,8 @@ use std::error::Error;
 use std::fmt;
 use std::mem;
 
+use crate::links::Links;
 use crate::non_max::NonMaxUsize;
-use crate::tree::Links;
 use crate::Kind;
 use crate::Span;
 use crate::Tree;
@@ -25,7 +25,7 @@ pub struct Id(NonMaxUsize);
 /// ```
 /// use syntree::{CloseError, Span, TreeBuilder};
 ///
-/// # fn main() -> anyhow::Result<()> {
+/// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let mut tree = TreeBuilder::new();
 ///
 /// tree.open("root");
@@ -61,7 +61,7 @@ impl fmt::Display for CloseError {
 ///     Lit,
 /// }
 ///
-/// # fn main() -> anyhow::Result<()> {
+/// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let mut tree = TreeBuilder::new();
 ///
 /// tree.open(Syntax::Number);
@@ -102,7 +102,7 @@ impl fmt::Display for BuildError {
 ///     Child,
 /// }
 ///
-/// # fn main() -> anyhow::Result<()> {
+/// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let mut tree = TreeBuilder::new();
 ///
 /// tree.open(Syntax::Root);
@@ -117,7 +117,7 @@ impl fmt::Display for BuildError {
 /// let expected = syntree::tree! {
 ///     Syntax::Root => {
 ///         Syntax::Child,
-///         Syntax::Child,
+///         Syntax::Child
 ///     }
 /// };
 ///
@@ -127,10 +127,10 @@ impl fmt::Display for BuildError {
 #[derive(Debug)]
 pub struct TreeBuilder<T> {
     /// Data in the tree being built.
-    pub(crate) data: Vec<Links<T>>,
-    /// Nodes currently being built.
-    stack: Vec<NonMaxUsize>,
-    /// The last sibling inserted.
+    tree: Tree<T>,
+    /// References to parent nodes of the current node being constructed.
+    parents: Vec<NonMaxUsize>,
+    /// Reference to last sibling inserted.
     sibling: Option<NonMaxUsize>,
     /// The current cursor.
     cursor: usize,
@@ -151,7 +151,7 @@ impl<T> TreeBuilder<T> {
     ///     Child,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
     ///
     /// tree.open(Syntax::Root);
@@ -167,8 +167,8 @@ impl<T> TreeBuilder<T> {
     /// ```
     pub const fn new() -> Self {
         TreeBuilder {
-            data: Vec::new(),
-            stack: Vec::new(),
+            tree: Tree::new(),
+            parents: Vec::new(),
             sibling: None,
             cursor: 0,
         }
@@ -191,7 +191,7 @@ impl<T> TreeBuilder<T> {
     ///     Child,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
     ///
     /// tree.open(Syntax::Root);
@@ -207,7 +207,7 @@ impl<T> TreeBuilder<T> {
     /// ```
     pub fn open(&mut self, data: T) -> Id {
         let id = self.insert(data, Kind::Node, Span::point(self.cursor));
-        self.stack.push(id);
+        self.parents.push(id);
         Id(id)
     }
 
@@ -229,7 +229,7 @@ impl<T> TreeBuilder<T> {
     ///     Child,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
     ///
     /// tree.open(Syntax::Root);
@@ -244,18 +244,18 @@ impl<T> TreeBuilder<T> {
     /// # Ok(()) }
     /// ```
     pub fn close(&mut self) -> Result<(), CloseError> {
-        let head = match self.stack.pop() {
+        let head = match self.parents.pop() {
             Some(head) => head,
             None => return Err(CloseError),
         };
 
         self.sibling = Some(head);
 
-        if let Some(parent) = self.stack.last().copied() {
-            if let Some(node) = self.data.get_mut(head.get()) {
+        if let Some(parent) = self.parents.last().copied() {
+            if let Some(node) = self.tree.get_mut(head) {
                 let end = node.span.end;
 
-                if let Some(parent) = self.data.get_mut(parent.get()) {
+                if let Some(parent) = self.tree.get_mut(parent) {
                     parent.span.end = end;
                 }
             }
@@ -280,7 +280,7 @@ impl<T> TreeBuilder<T> {
     ///     Number,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
     ///
     /// tree.open(Syntax::Child);
@@ -291,9 +291,8 @@ impl<T> TreeBuilder<T> {
     /// ```
     pub fn token(&mut self, data: T, len: usize) -> Id {
         let start = self.cursor;
-        let end = self.cursor + len;
-        self.cursor = end;
-        let id = self.insert(data, Kind::Token, Span::new(start, end));
+        self.cursor = self.cursor.checked_add(len).expect("cursor out of bounds");
+        let id = self.insert(data, Kind::Token, Span::new(start, self.cursor));
         self.sibling = Some(id);
         Id(id)
     }
@@ -308,46 +307,47 @@ impl<T> TreeBuilder<T> {
     /// # Examples
     ///
     /// ```
-    /// use anyhow::Result;
-    /// use syntree::{print, Span, TreeBuilder};
+    /// use syntree::TreeBuilder;
     ///
     /// #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
     /// enum Syntax {
-    ///     Root,
-    ///     Number,
-    ///     Lit,
-    ///     Whitespace,
+    ///     ROOT,
+    ///     NUMBER,
+    ///     LIT,
+    ///     WHITESPACE,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// use Syntax::*;
+    ///
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut b = TreeBuilder::new();
     ///
     /// let c = b.checkpoint();
     ///
-    /// b.open(Syntax::Number);
-    /// b.token(Syntax::Lit, 1);
+    /// b.open(NUMBER);
+    /// b.token(LIT, 1);
     /// b.close()?;
     ///
-    /// b.token(Syntax::Whitespace, 3);
+    /// b.token(WHITESPACE, 3);
     ///
-    /// b.open(Syntax::Number);
-    /// b.token(Syntax::Lit, 2);
-    /// b.token(Syntax::Lit, 2);
+    /// b.open(NUMBER);
+    /// b.token(LIT, 2);
+    /// b.token(LIT, 2);
     /// b.close()?;
     ///
-    /// b.close_at(c, Syntax::Root);
+    /// b.close_at(c, ROOT);
     ///
     /// let tree = b.build()?;
     ///
-    /// let root = tree.first().unwrap();
-    /// assert_eq!(*root.data(), Syntax::Root);
+    /// let root = tree.first().ok_or("missing root")?;
+    ///
+    /// assert_eq!(*root.data(), ROOT);
     /// assert_eq!(root.children().count(), 3);
     /// assert_eq!(root.children().without_tokens().count(), 2);
     /// # Ok(()) }
     /// ```
     pub fn checkpoint(&self) -> Id {
-        let id = NonMaxUsize::new(self.data.len()).expect("ran out of ids");
-        Id(id)
+        Id(NonMaxUsize::new(self.tree.len()).expect("ran out of ids"))
     }
 
     /// Insert a node that wraps from the given checkpointed location.
@@ -361,7 +361,7 @@ impl<T> TreeBuilder<T> {
     /// ```
     /// use syntree::TreeBuilder;
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut a = TreeBuilder::<u32>::new();
     /// a.open(0);
     /// let c = a.checkpoint();
@@ -399,7 +399,7 @@ impl<T> TreeBuilder<T> {
     ///
     /// use Syntax::*;
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut b = TreeBuilder::new();
     ///
     /// let c = b.checkpoint();
@@ -436,9 +436,9 @@ impl<T> TreeBuilder<T> {
     pub fn close_at(&mut self, id: Id, data: T) -> Id {
         // With the layout of this data structure this is a fairly simple
         // operation.
-        let child = NonMaxUsize::new(self.data.len()).expect("ran out of ids");
+        let child = NonMaxUsize::new(self.tree.len()).expect("ran out of ids");
 
-        let links = match self.data.get_mut(id.0.get()) {
+        let links = match self.tree.get_mut(id.0) {
             Some(links) => links,
             None => {
                 let id = self.insert(data, Kind::Node, Span::point(self.cursor));
@@ -459,22 +459,19 @@ impl<T> TreeBuilder<T> {
         );
 
         // Adjust span to encapsulate all children.
-        let mut span = links.span;
+        let start = links.span.start;
 
-        if let Some(mut cur) = removed.next.and_then(|id| self.data.get(id.get())) {
-            span = span.join(cur.span);
+        if let Some(node) = self.tree.node_at(removed.next) {
+            if let Some(last) = node.children().last() {
+                let span = Span::new(start, last.span().end);
 
-            while let Some(next) = cur.next.and_then(|id| self.data.get(id.get())) {
-                span = span.join(next.span);
-                cur = next;
-            }
-
-            if let Some(node) = self.data.get_mut(id.0.get()) {
-                node.span = span;
+                if let Some(node) = self.tree.get_mut(id.0) {
+                    node.span = span;
+                }
             }
         }
 
-        self.data.push(removed);
+        self.tree.push(removed);
 
         // The current sibling is the newly replaced node in the tree.
         self.sibling = Some(id.0);
@@ -497,7 +494,7 @@ impl<T> TreeBuilder<T> {
     ///     Number,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
     ///
     /// tree.open(Syntax::Child);
@@ -530,7 +527,7 @@ impl<T> TreeBuilder<T> {
     ///     Lit,
     /// }
     ///
-    /// # fn main() -> anyhow::Result<()> {
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
     ///
     /// tree.open(Syntax::Number);
@@ -543,25 +540,22 @@ impl<T> TreeBuilder<T> {
     /// assert!(matches!(tree.build(), Err(BuildError { .. })));
     /// # Ok(()) }
     /// ```
-    pub fn build(self) -> Result<Tree<T>, BuildError>
-    where
-        T: Clone,
-    {
-        if !self.stack.is_empty() {
+    pub fn build(self) -> Result<Tree<T>, BuildError> {
+        if !self.parents.is_empty() {
             return Err(BuildError);
         }
 
-        Ok(Tree::new(self.data.into()))
+        Ok(self.tree)
     }
 
     /// Insert a new node.
     fn insert(&mut self, data: T, kind: Kind, span: Span) -> NonMaxUsize {
-        let new = NonMaxUsize::new(self.data.len()).expect("ran out of ids");
+        let new = NonMaxUsize::new(self.tree.len()).expect("ran out of ids");
 
         let prev = std::mem::replace(&mut self.sibling, None);
-        let parent = self.stack.last().copied();
+        let parent = self.parents.last().copied();
 
-        self.data.push(Links {
+        self.tree.push(Links {
             data,
             kind,
             next: None,
@@ -569,11 +563,11 @@ impl<T> TreeBuilder<T> {
             span,
         });
 
-        if let Some(node) = prev.and_then(|id| self.data.get_mut(id.get())) {
+        if let Some(node) = self.tree.links_at_mut(prev) {
             node.next = Some(new);
         }
 
-        if let Some(node) = parent.and_then(|id| self.data.get_mut(id.get())) {
+        if let Some(node) = self.tree.links_at_mut(parent) {
             if node.first.is_none() {
                 node.first = Some(new);
             }
